#!/usr/bin/perl
###########################################################################
#    WWWThreads  - Interactive Discussion Forum
#    Rick Baker scream@screamingweb.com 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
###########################################################################
  use w3tvars qw(%config);
  use w3t qw($dbh);

  use strict;

# -----------------------
# Connect to the database
   w3t::db_connect();

# -------------
# Get the input
  my %FORM = w3t::get_input;


# --------------------
# Assign the variables
  my $Username = $FORM{'Username'};
  my $Email    = $FORM{'Email'};
  my $Verify   = $FORM{'Verify'};
  my $sth;

# -----------------------------------------------
# Don't let them register the Username Anonymous
  if( ($Username eq "Anonymous") || ($Username eq "anonymous") ) {
    w3t::not_right("You cannot register the username Anonymous");
  }


# ------------------------------------------------------------
# If all required info is not filled in, then we can't proceed 
  if((!$Username)||(!$Email)){

    w3t::not_right("All of the required information has not been filled in.  Please try again.");

  }


# -------------------------------------------------------------
# If the Username isn't the proper length then we can't proceed  
  if((length($Username) >16) || (length($Username) < 3)) {

    w3t::not_right("Username is not the proper length.  Please try again.");

  }




# --------------------------------------------
# Check to make sure the Username is available
  my $Username_q = $dbh -> quote ($Username);
  my $query = qq!
    SELECT Username
    FROM   Users
    WHERE  Username = $Username_q
  !;

  $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
  
# ----------------------------------------------------------
# If sql returned a row then that username is already in use.
  if( $sth -> rows ) {

    $sth -> finish;
    w3t::not_right("That Username is already in use.  Please try again.");

  }
  $sth -> finish;

# ---------------------------------------------------------------------
# If we do not allow multiple usernames for the same email address then
# we need to see if this email address is in the database
  if ($config{'multiuser'} eq "off" ) {
    my $Email_q = $dbh -> quote($Email);
    $query = qq!
      SELECT Email
      FROM Users
      WHERE Email = $Email_q
    !;

    $sth = $dbh -> prepare($query) or die "Can't prepare $query.  Reason: $DBI::errstr.";
    $sth -> execute() or die "Can't execute $query.  Reason: $DBI::errstr.";

    if($sth -> rows){

      $sth -> finish;
      w3t::not_right("Multiple accounts are not allowed for a single email address and we already have a Username for the email address you provided.");

    }
    $sth -> finish;

  } 


# --------------------------------------
# Check to see if this is the first user
  $query = qq!
    SELECT Username
    FROM   Users
  !;
  
  $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
  my $Status = "";
  my $Security = $config{'user_security'};
  my $rows = $sth -> rows;
  $sth -> finish;

# -------------------------------------------------------
# If this is the first user, then status is Administrator
# otherwise they are just get normal user status.
  if (!$rows){

    $Status = "Administrator";
    $Security = 100;

  } else {

    $Status = "User";

  } 


# ------------
# Get the date
  my $date = w3t::get_date;


# ------------------------------
# Put the user into the database
  my $Status_q   = $dbh -> quote($Status);
  $Username_q    = $dbh -> quote($Username);
  my $Email_q    = $dbh -> quote($Email);
  my $Display_q  = $dbh -> quote($config{'postlist'});
  my $View_q     = $dbh -> quote($config{'threaded'}); 
  my $EReplies_q = $dbh -> quote("Off");
  $query = qq!
    INSERT INTO Users (Username,Email,Totalposts,Laston,Status,Sort,Display,View,PostsPer,EReplies,Security,Registered)
    VALUES ($Username_q,$Email_q,0,$date,$Status_q,$config{'sort'},$Display_q,$View_q,$config{'postsperpage'},$EReplies_q,$Security,$date)
  !;

  $dbh -> do($query) or die "Can't execute query: $query. Reason: $DBI::errstr";


# ----------------------------------
# Grab all the boards in the database 
  $query = qq!
    SELECT Keyword 
    FROM Boards
  !;
  $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";

  $rows = $sth->rows;

# ---------------------------------------------------
# Now we put this user into each last viewed database
  my $curr = w3t::get_date();
  for (my $i=0; $i<$rows; $i++){

    my $Lastviewed = $sth -> fetchrow_array."Last";

    # --------------------------------------------------------------------
    # We have to get rid of any exisiting entries because in versions
    # 3.4, 3.4.1 and 3.4.2 I forgot to remove these entries when deleting
    # a user.
    $query = qq!
      DELETE FROM $Lastviewed
      WHERE Username = $Username_q
    !;
    $dbh -> do($query) or die "Can't execute query: $query. Reason: $DBI::errstr";
    $query = qq!
      INSERT INTO $Lastviewed (Username,Last) VALUES ($Username_q,$curr)
    !;
    $dbh -> do($query) or die "Can't execute query: $query. Reason: $DBI::errstr";

  }
  $sth -> finish;

# --------------------
# Grab welcome.include  
  open(FILE,"$config{'includes'}/welcome.include") or die "Can't open file '$config{'includes'}/welcome.include'";
  my $Message = '';
  while(<FILE>){
    $Message = $Message."$_";
  }
  my $Message_q = $dbh -> quote($Message);
  close(FILE);


# --------------------------------
# Get the Administrator's Username
  my $Admin = "Administrator";
  my $Admin_q = $dbh -> quote($Admin);
  
  $query = qq!
    SELECT Username
    FROM   Users
    WHERE  Status = $Admin_q
  !;
  $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";



# ---------------------------------
# Set up some stuff for the message
  my ($Sender)   = $sth -> fetchrow_array;
  $sth -> finish;
  my $Sender_q   = $dbh -> quote ($Sender);
  my $Messstat   = "N";
  my $Messstat_q = $dbh -> quote($Messstat);
  my $Subject    = "Welcome";
  my $Subject_q  = $dbh -> quote($Subject);


# --------------------------------------
# Put the message into the Messages table
  $query = qq!
    INSERT INTO Messages (Username,Status,Subject,Sender,Message,Sent)
    VALUES ($Username_q,$Messstat_q,$Subject_q,$Sender_q,$Message_q,$date)
  !;
  $dbh -> do($query) or die "Can't execute query: $query. Reason: $DBI::errstr";


# -----------------------------------------
# Now we need to generate a random password
  my $a = time();
  srand($a);
  my @passet = ('a'..'k', 'm'..'z', 'A'..'N', 'P'..'Z', '2'..'9');
  my $rnd_passwd;
  my $pass;
  for (my $i = 0; $i < 6; $i++) {
    my $random_num = int(rand($#passet + 1));
    $pass .= @passet[$random_num];
  }

# ----------------------------
# Now let's crypt the password
  my $a = time();
  srand($a);
  my $one   = int(rand($#passet)) + 1;
  my $two   = int(rand($#passet)) + 1;
  my $salt  = "@passet[$one]"."@passet[$two]";
  my $crypt = crypt($pass,$salt);


# -----------------------------
# Now let's update the database
  my $Username_q = $dbh -> quote($Username);
  my $crypt_q    = $dbh -> quote($crypt);
  my $query = qq!
    UPDATE Users SET Password = $crypt_q
    WHERE Username = $Username_q
  !;
  $dbh -> do($query) or die "Can't execute $query.  Reason: $DBI::errstr";



# -------------------------------------
# Now we need to mail them the password
  require Mailer;
  my $ip = $ENV{'REMOTE_ADDR'};

  my $query = qq!
    SELECT Username,Email
    FROM   Users
    WHERE  Username = $Username_q
  !;
  my $sth = $dbh -> prepare($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute $query.  Reason: $DBI::errstr.";

  if ($sth -> rows) {

    my ($Username,$Email) = $sth -> fetchrow_array;
    my $mailer = new Mailer ( { smtp => $config{'SMTP'}, sendmail => $config{'sendmail'} } ) or die "Can't init: $Mailer::error";
    my $to      = $Email;
    my $from    = $config{'emailaddy'};
    my $subject = "Your password for $config{'title'}";
    my $msg     = "Someone from the ip address '$ip' registered the Username '$Username'.  The password for this Username is '$pass'.";

    $mailer -> send( {to => $to, from => $from, subject => $subject, msg => $msg} ) or die "Can't mail message: $Mailer::error";

  }
  $sth -> finish;

# ------------------------
# Send them a confirmation
  w3t::send_header("Username has been registered");
  w3t::table_header("Username has been registered");

  print qq~ 

  The Username '$Username' has been reserved.  You should be receiving an email shortly with your password.  Once you receive your password you can <a href="$config{'cgiurl'}/login.pl">Login</a> and post with this Usename.  Thanks for registering a Username.  Please take the time to edit your profile before posting on the forums.  Editing your profile will allow you to customize the layout of the forums to suit your particular tastes and needs.  
  </P>

  ~;


# -------------
# Send a footer
  w3t::send_footer();

