#!/usr/bin/perl 
###########################################################################
#    WWWThreads  - Interactive Discussion Forum
#    Rick Baker scream@screamingweb.com 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
###########################################################################
  use w3tvars qw(%config);
  use w3t qw($dbh);
  use strict;


# ----------------------
# Connect to the database
  w3t::db_connect();


# --------------------------
# Make sure this is an admin!
  my %user = w3t::authenticate();
  if ($user{'Status'} ne 'Administrator') {
    w3t::not_right ("You must be logged in, and be a valid administrator to access this.");
  }


# -------------
# Get the input
  my %FORM = w3t::get_input();


# --------------------
# Assign the variables
  my $Title       = $FORM{'Title'};
  my $Keyword     = $FORM{'Keyword'};
  my $Description = $FORM{'Description'};
  my $HTML        = $FORM{'HTML'};
  my $Markup      = $FORM{'Markup'};
  my $Anonymous   = $FORM{'Anonymous'};
  my $Locked      = $FORM{'Locked'};
  my $Expires     = $FORM{'Expires'};
  my $Security    = $FORM{'Security'};
  my $Category    = $FORM{'Category'};


# ------------
# Set Defaults.
  $HTML      ||= "Off";
  $Markup    ||= "Off";
  $Anonymous ||= "Off";
  $Locked    ||= "Open";

# -----------------------------------------
# Make sure we have a proper security level
  if ( ($Security < 1) || ($Security > 100) ) {
    w3t::not_right("The security level must be between 1 and 100.");
  }

# ---------------------------------------------
# Check to make sure all info has been filled in
  if((!$Title)||(!$Keyword)||(!$Description)){
    w3t::not_right("All of the required informatin has not been filled in.  Please try again.");
  }


# ---------------------
# Grab the current time
  my $date = w3t::get_date();


# -------------------------------
# Put the board into the database
  my $Title_q       = $dbh -> quote($Title);
  my $Description_q = $dbh -> quote($Description);
  my $Username_q    = $dbh -> quote($user{'Username'});
  my $HTML_q        = $dbh -> quote($HTML);
  my $Markup_q      = $dbh -> quote($Markup);
  my $Anonymous_q   = $dbh -> quote($Anonymous);
  my $Locked_q      = $dbh -> quote($Locked);
  my $Keyword_q     = $dbh -> quote($Keyword);
  my ($Cat,$Catnum) = split(/-PARSER-/,$Category);
  my $Category_q    = $dbh -> quote($Cat);

  my $query = qq!
    INSERT INTO Boards (Title,Description,Keyword,Moderator,Total,Last,HTML,Anonymous,Locked,Created,Expire,Security,Markup)
    VALUES ($Title_q, $Description_q, $Keyword_q,$Username_q,0,0,$HTML_q,$Anonymous_q,$Locked_q,$date,$Expires,$Security,$Markup_q)
  !;
  $dbh->do ($query) or w3t::not_right("That keyword is already being used.  Please try again.");


# ----------------------------------
# Insert it into the proper category
  my $Keyword_q  = $dbh -> quote($Keyword);
  $query = qq!
    INSERT INTO Category (Title,Number,Keyword)
    VALUES ($Category_q,$Catnum,$Keyword_q)
  !;
  $dbh->do ($query) or w3t::not_right("That keyword is already being used.  Please try again.");

# -------------------------
# Create the keyword's table
  $query = qq!
    CREATE TABLE $Keyword (
      Number INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
      Parent INT(11) UNSIGNED NOT NULL,
      Main INT(11) UNSIGNED NOT NULL,
      Posted INT(11) UNSIGNED NOT NULL,
      Last_Post int(11) unsigned DEFAULT '0' NOT NULL,
      Username CHAR(30),
      IP CHAR(60),
      Subject TEXT,
      Body TEXT,
      Mail INT(1),
      File CHAR(100),
      Kept CHAR(1) NOT NULL,
      Status Char(1) NOT NULL,
      INDEX indx1 (Number),
      INDEX indx2 (Parent),
      INDEX indx3 (Main),
      INDEX indx4 (Kept),
      INDEX indx5 (Status),
      INDEX indx6 (Last_Post),
      INDEX indx7 (Posted)
  )
  !;
  $dbh->do ($query) or die "Can't create board: $query. Reason: $DBI::errstr";


# --------------------------
# Create the last read table
  my $Last = "$Keyword"."Last";
  $query = qq!
    CREATE TABLE $Last (
      Username CHAR(30) PRIMARY KEY,
      Last INT(11)
  )
  !;
  $dbh->do ($query) or die "Can't create board: $query. Reason: $DBI::errstr";



# ------------------------
# Send them a confirmation
  w3t::send_header ("The new board has been created.","<META HTTP-EQUIV=\"Refresh\" content=\"5;url=$config{'cgiurl'}/admin/login.pl\">");
  w3t::table_header("The new board has been created.");

  print " The new board has been created and is now available for use by the users.  </P>";
  w3t::send_footer();
