#!/usr/bin/perl
###########################################################################
#    WWWThreads  - Interactive Discussion Forum
#    Rick Baker scream@screamingweb.com 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
###########################################################################
package w3t;

require Exporter;
@ISA = qw (Exporter);
@EXPORT = qw(@EXPORT_OK);

@EXPORT_OK = qw($dbh $VERSION);

use strict;
use w3tvars qw(%config);
use vars qw($dbh $VERSION); 

$w3t::VERSION = "3.5.1";

# Some versions of perl are barking about the DBI and statement handlers
# For now, we are just turning these off.
#$SIG{'__WARN__'} = sub { };
###########################################################################
# Start of the main subroutines
###########################################################################


###########################################################################
# Get_input - Parses form input and returns it in a hash.
# Useage: my %in = &get_input;
###########################################################################
sub get_input {

  my $input = '';
  my %in    = ();
  ($ENV{'REQUEST_METHOD'} eq "POST") and  read(STDIN, $input, $ENV{'CONTENT_LENGTH'});
  ($ENV{'REQUEST_METHOD'} eq "GET")  and  ($input = $ENV{'QUERY_STRING'});

  my @pairs = split(/&/, $input);
  foreach (@pairs) {
    my ($name, $value) = split /=/;
  # Un-Webify plus signs and %-encoding
    $value =~ tr/+/ /;
    $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
    $value =~ s/<!--(.|\n)*-->//g;
    $value =~ s/\n/<br>/g;
    $value =~ s/\r//g;
    $in{$name} = $value;
  }
  return %in;

}


##############################################################
# Get File - Will process multipart form-data and return the
# results.
# Created on Mon Jul 22 12:18:15 PDT 1996 by Steve Hsueh
# Modified by Rick Baker to loop through the entire input and
# return normal form data as well.
# Currently this only works with Mozilla 4+ browsers so we
# always run browser_check to see if this is used or not.
# This entire sub is begging for someone who knows what the
# heck they are doing to rewrite it :).  It would be nice
# if it worked on all browser but for now it will have to
# make due.
##############################################################
sub get_file {

  my ($i, $loc, $key, $val, $input);
  my (%ATTACH, $f,$header, $header_body, $len, $buf);

  if ($ENV{'REQUEST_METHOD'} eq "GET")
  { $input = $ENV{'QUERY_STRING'}; }
  elsif ($ENV{'REQUEST_METHOD'} eq "POST") {
    $len = 0;
    $input = '';
    while( $len != $ENV{'CONTENT_LENGTH'} ) {
      $buf = '';
      $len += sysread(STDIN, $buf, $ENV{'CONTENT_LENGTH'});
      $input .= $buf;
    }
  }

  if( $ENV{'CONTENT_TYPE'} =~ /multipart\/form-data; boundary=(.+)$/ ) {
    my $boundary = '--'.$1;  
    my @list = split(/$boundary/, $input);
  # For some reason there are always 2 extra, that are empty
    my $size = @list -2 ;

    for (my $x=1;$x<=$size;$x++){
      $header_body = $list[$x];
      $header_body =~ /\r\n\r\n|\n\n/; 
    # Here we split the header and body
      $header = $`;   
      my $body   = $'; 
      $body =~ s/\r\n$//;
    # Now we try to get the file name
      my $name = $header;
      my $blah;
      $name =~ /name=\"(.+)"/;
      ($name,$blah) = split(/"/,$1);
      $ATTACH{'NAME'} = $name;
      $ATTACH{'FILE_CONTENT'} = $body;
    # If the form name is not attach, then we need to parse this like
    # regular form data 
      if ($name ne "attach") {
        $body =~ tr/+/ /;
        $body =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
        $body =~ s/<!--(.|\n)*-->//g;
        $body =~ s/\n/<br>/g;
        $body =~ s/\r//g;
        $ATTACH{$name} = $body;
    # Otherwise it is an attachment and we need to finish it up
      }else{

        $header =~ /filename=\"(.+)\"/;
        $ATTACH{'FILE_NAME'} = $1;
        $ATTACH{'FILE_NAME'} =~ s/\"//g;
        $ATTACH{'FILE_NAME'} =~ s/\s//g;

        for( $i=$x; $list[$i]; $i++) {
          $list[$i] =~ s/^.+name=$//;
          $list[$i] =~ /\"(\w+)\"/;
          $ATTACH{$1} = $';
        }
      }
    }

    return %ATTACH;

  }

  return %ATTACH; 

}


#######################################################################
# Browser check - Checks to see if the browser is Mozilla 4 or greater
# Returns true if it is
#######################################################################

sub browser_check{

  my $browser = $ENV{'HTTP_USER_AGENT'};
  my $version;
  my $check;
  my $stuff;
  if ($browser =~/Mozilla/){
    ($browser,$version) = split(/Mozilla\//,$browser);
    ($version,$stuff)   = split(/ /,$version);
  }
  if ($version >= 4) {
    $check = "true";
  }

  return $check;

} 


###########################################################################
# url_encode: Encodes a string for printing in a URL.
# Usage: my $encoded = &url_encode ($string);
###########################################################################
sub url_encode {

  my $input = shift;
  $input =~s/([^a-zA-Z0-9_.-])/uc sprintf("%%%02x",ord($1))/eg;
  return $input

}

 
###########################################################################
# Url_decode: Decodes a string that's been encoded in a URL.
# Usage: my $string = &url_decode ($encoded);
###########################################################################
sub url_decode {

  my $input = shift;
  $input =~ tr/+/ /;
  $input =~ s/%([0-9a-fA-F]{2})/pack("c",hex($1))/ge;
  return $input;

}


###########################################################################
# check_refer: Checks for a valid referer
###########################################################################
sub check_refer {

  my $valid = 0;
  ($ENV{'HTTP_REFERER'} =~ /$config{'referer'}/i) and $valid++;
    
  $valid or w3t::not_right ("The host you are trying to send the input from is not a valid host.");

}


###########################################################################
# get_date: returns a date in seconds
###########################################################################
sub get_date {

  my ($currtime)=time();
  my ($adjust)=$currtime+($config{'adjusttime'}*3600);
  return $adjust;

}


###########################################################################
# convert_time: converts the time to a useable format
###########################################################################
sub convert_time {

  my $time = shift; 
  $time = localtime($time);
  if($config{'timeformat'} eq "short") {

    my @tarray = split(/ +/,$time);
    my %months = (
      "Jan" => 1,
      "Feb" => 2,
      "Mar" => 3,
      "Apr" => 4,
      "May" => 5,
      "Jun" => 6,
      "Jul" => 7,
      "Aug" => 8,
      "Sep" => 9,
      "Oct" => 10,
      "Nov" => 11,
      "Dec" => 12
    );         
    my $year = substr($tarray[4],2,2);
    my ($hour,$min,$sec) = split(/:/,$tarray[3]);
    my $AMPM;
    if ($hour >= 12) {
      $AMPM = "PM";
      if ($hour > 12) {
        $hour = $hour-12;
      }
    }
    else {
      $AMPM = "AM";
    }

    $time = "$months{$tarray[1]}/$tarray[2]/$year $hour:$min $AMPM";

  }
  return $time;

}


###########################################################################
# Connects to the database.
###########################################################################
sub db_connect{

  if (!$ENV{'MOD_PERL'}) {
    require DBI;
  }

  $dbh = DBI -> connect("DBI:$config{'dbdriver'}:$config{'dbname'}:$config{'dbserver'}", "$config{'dbuser'}", "$config{'dbpass'}")
  or die "Can't connect to sql server.  Reason: $DBI::errstr";

  if (!$ENV{'MOD_PERL'}) {
    $dbh->{'Warn'} = 0;
  }

}


###########################################################################
# Disconnects from the database
###########################################################################
sub db_disconnect{

   defined $dbh and
    ($dbh->disconnect or die "Can't disconnect from database. Reason: $DBI::errstr" and undef $dbh);

}

###########################################################################
# Get_cookie - Gets all the cookies and returns them in a hash
# Usage: my %cookie = &get_cookie;
###########################################################################
sub get_cookie {

  my @cstuff = @_;
  my ($cookie, $value, $char, %cookie);

  my @Cdec = ('\+', '\%3A\%3A', '\%3D', '\%2C', '\%25', '\%2B', '\%26','\%3B');
  my %Cdec = ('\+',' ','\%3A\%3A','::','\%3D','=','\%2C',',','\%25','%','\%2B','+','\%26','&','\%3B',';');

  if ($ENV{'HTTP_COOKIE'}) {

    foreach (split(/; /,$ENV{'HTTP_COOKIE'})) {

    ($cookie,$value) = split(/=/);

      foreach $char (@Cdec) {

        $cookie =~ s/$char/$Cdec{$char}/g;
        $value =~ s/$char/$Cdec{$char}/g;

      }

      $cookie{$cookie} = $value;

    }

  }

  return %cookie;

}

###########################################################################
# set_cookie - Sets a cookie.
# Usage : set_cookie('Name',"Value",?) ? = 0 or 1. 0 = temp, 1 = permanent
###########################################################################
sub set_cookie {

  my @cookie = @_;
  my ($cookie, $value, $type, $char);    
  my @Cenc = ('\;','\&','\+','\%','\,','\=','\:\:','\s');
  my %Cenc = ('\;','%3B','\&','%26','\+','%2B','\%','%25','\,','%2C','\=','%3D','\:\:','%3A%3A','\s','+');

  my $header = '';

  for (my $i = 0; $i <= $#cookie; $i = $i + 3) {
    ($cookie, $value, $type) = @cookie[$i .. $i+2];
    foreach $char (@Cenc) {
      $cookie =~ s/$char/$Cenc{$char}/g;
      $value =~ s/$char/$Cenc{$char}/g;
    }
    $header = 'Set-Cookie: ' . $cookie . '=' . $value . ';';
    if ($type == 1) {
      $header .= ' expires=' . $config{'cexp'} . ';'
    }
    print "$header\n";
  }

}


###########################################################################
# Figure out how we should exit
###########################################################################
sub exit{

  if(exists $ENV{MOD_PERL}) {
    Apache::exit(0); 
  }
  else {
    CORE::exit(0);
  }

}


########################################################################
sub authenticate {
# ---------------------------------------------------------
# Checks the username/password to make sure it is valid, and
# returns a hash of the user info. If no parameters are provided,
# it gets the username/password from cookies. Returns undef if unable
# to validate or no username/password found.
#
	my ($Username, $Password);
	if (@_ == 2) {
		($Username, $Password) = @_;
	}
	else {
		my %cookies = &get_cookie();
		$Username   = $cookies{'Username'};
		$Password   = $cookies{'Password'};
	}
	return undef unless $Username and $Password;

	my $Username_q  = $dbh->quote($Username);
        my $Password_q  = $dbh->quote($Password);
    
	my $query = qq!
		SELECT * 
		FROM   Users
		WHERE  Username = $Username_q
	!;
	my $sth = $dbh->prepare ($query) or die "Can't prepare: $query. Reason: $!";
	$sth->execute                    or die "Can't execute: $query. Reason: $!";

	if ($sth->rows) {
          my $user = $sth->fetchrow_hashref;
          $sth->finish;
          return %{$user};
	}
	$sth->finish;
	return undef;
}

######################################################################
# send_message - Sends a message to a user
######################################################################

sub send_message{

  w3t::db_connect();

  my ($Sender,$To,$Subject,$Mess) = @_;

  my $Sender_q  = $dbh -> quote($Sender);
  my $To_q      = $dbh -> quote($To);
  my $Subject_q = $dbh -> quote($Subject);
  my $Mess_q    = $dbh -> quote($Mess);
  my $date      = w3t::get_date();
  my $Status_q  = $dbh->quote("N");


# ----------------------------------------------
# Find out if we are sending to a group of users
  my $selector = '';
  my $admin_q  = $dbh -> quote("Administrator");
  my $mod_q    = $dbh -> quote("Moderator"); 
  my $user_q   = $dbh -> quote("User"); 

  if ($To eq "ADMIN_GROUP") {
    $selector = "WHERE Status = $admin_q";
  }
  elsif ($To eq "MODERATOR_GROUP") {
    $selector = "WHERE Status = $mod_q";
  }
  elsif ($To eq "A_M_GROUP") {
    $selector = "WHERE Status = $admin_q OR Status = $mod_q";
  }
  elsif ($To eq "USER_GROUP") {
    $selector = "WHERE Status = $user_q";
  }
  elsif ($To eq "ALL_USERS") {
    $selector = ""; 
  }
  else {
    $selector = "WHERE Username = $To_q";
  }


# --------------------------
# Lets get who we are sending this to 
  my $query = qq!
    SELECT DISTINCT Username,Status
    FROM Users
    $selector 
  !;
  my $sth = $dbh -> prepare($query) or die "Query syntax error: $query. $DBI::errstr.";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr.";
  my $rows = $sth -> rows;
  if ($rows < 1) {
    return;
  }

  for (my $i = 0; $i<$rows; $i++){ 

    my ($To,$Status)   = $sth -> fetchrow_array;
    my $To_q = $dbh -> quote($To);

    my $query = qq!
      INSERT INTO Messages (Username,Status,Subject,Sender,Message,Sent)
      VALUES ($To_q,$Status_q,$Subject_q,$Sender_q,$Mess_q,$date)
    !;
    $dbh -> do($query) or die "Can't execute query: $query. Reason: $DBI::errstr";

  }

}

#####################################################################
# check for banned users or hostnames
#####################################################################

sub check_ban{

  my $Username = shift;
 
  my $Hostname = $ENV{'REMOTE_ADDR'};
 
  $Username ||= " ";
  $Hostname ||= " ";
  my $Username_q = $dbh -> quote($Username);
  my $Hostname_q = $dbh -> quote($Hostname);

  my $query = qq!
    SELECT Reason
    FROM Banned
    WHERE Username = $Username_q OR Hostname = $Hostname_q
  !;
  my $sth = $dbh -> prepare($query) or die "Query syntax error: $query. Reason:$DBI::errstr.";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";

  my $rows = $sth -> rows;
  if ($rows > 0) {
  
    my ($Reason) = $sth -> fetchrow_array;
    $sth -> finish;
    w3t::not_right("You have been banned from making any new posts or sending private messages.  The reason for this ban is: $Reason");

  }

}


###########################################################################
# The following subs define the looks and feel of the program
###########################################################################

###########################################################################
# Prints out the beginning HTML.  Pass this $title and $refresh.
###########################################################################
sub send_header{

  my ($title, $refresh,$Username,$Password,$bypass) = @_;

# ---------------------
# Let's grab their info 
#  w3t::db_connect();

  my %user;
  if (($Username) && ($Password)){
    %user = w3t::authenticate($Username,$Password);
  }
  else {
    %user = w3t::authenticate();
  }
  print "Content-type: text/html\n\n";
  print "<HTML><HEAD>";
  $refresh ||="";
  if (!($refresh eq "bypass")){
    print $refresh;
  }
# ------------------------------------------------------
# Here is where the style sheet gets put into the header
  print"<STYLE TYPE=\"text/css\">\n";
 
  my $fontsize = "$config{'fontsize'}pt";
  my $fontface = $config{'fontface'};
  my $headerfont = $config{'headerfontface'};

  if ($user{'FontSize'}) {
    $fontsize = $config{'fontsize'} + $user{'FontSize'}."pt";
  }
  if ($user{'FontFace'}) {
    $fontface = $user{'FontFace'};
    $headerfont = $user{'FontFace'};
  }

  if ($config{'underline'} eq "no") {
    print"A:link, A:visited, A:active, A:hover { text-decoration: none; font-weight: $config{'linkweight'} }";
  }
  print qq~
      A:HOVER { color: "$config{'hovercol'}"; }

      BODY {
         font-size: $fontsize; 
         font-family: $fontface; 
         }  
      P {
         font-size: $fontsize; 
         font-family: $fontface; 
         }  
      TD {
         font-size: $fontsize; 
         font-family: $fontface; 
         }
      TD.header {
         font-size: $config{'headerfont'}pt; 
         font-family: $headerfont; 
         color: $config{'headerfontcolor'};
         }
      H3 {
         font-family: $fontface;
         } 
  ~;
  print "</STYLE>";
  print "<TITLE>$title - Powered by WWWThreads</TITLE></HEAD>";
  $config{'useimage'} ?
     print qq~<body background = "$config{'backimage'}" text = "$config{'textcol'}" link = "$config{'linkcol'}" vlink = "$config{'linkvis'}" alink = "$config{'linkactive'}">~ :
     print qq~<body bgcolor = "$config{'bodybg'}" text = "$config{'textcol'}" link = "$config{'linkcol'}" vlink = "$config{'linkvis'}" alink = "$config{'linkactive'}">~;

# ------------------------
# Include the header file.
  open(FILE,"$config{'includes'}/header.include") or &cgi_error("Unable to open header file: $config{'includes'}/header.include. Reason: $!");
  while(<FILE>) {

    print;

  }
  close(FILE);

  $bypass ||=0; 

# --------------------------------------
# Figure out where to position the menu
  if ($config{'menu'} eq "top") {  
    print "<table cellpadding=0 cellspacing=0 border=0 width=100%>";
    print "<tr bgcolor=$config{'tableheader'}><td align=right><font size=-1>";
  }
  else {
    print "<table border=0 width=100%><tr><td valign=top width=10%>";
    print "<table border=0><tr><td nowrap>";
  }

  if ( !($user{'Username'}) || ($bypass == 1) ) {

  # They aren't logged in so they get the standard menu
    open(FILE,"$config{'includes'}/menu.include") or &cgi_error("Unable to open menu file: $config{'includes'}/menu.include. Reason: $!");
    while (<FILE>) {

      s/_PARSER_/$config{'cgiurl'}/g;
      if ($config{'menu'} eq "top") {
        s/<br>/ | /g;
      }
      print $_;

    }


  } 
  else {

  # They are logged in so they get the special menu
   
  # --------------------------------------------------------------------------
  # If they are an admin or moderator then they get a link to the admin section
    if ( ($user{'Status'} eq "Administrator") || ($user{'Status'} eq "Moderator") ) {
      print "<a href = \"$config{'cgiurl'}/admin/login.pl\">Admin</a>";
      if ($config{'menu'} eq "side") {
        print "<br>";
      }
      else {
        print " | ";
      }

    }


    open(FILE,"$config{'includes'}/usermenu.include") or &cgi_error("Unable to open menu file: $config{'includes'}/menu.include. Reason: $!");
    while (<FILE>) {

      s/_PARSER_/$config{'cgiurl'}/g;
      if ( (!$config{'private'})  && (/Send Private/) ) {
        next;
      }

      if ($config{'menu'} eq "top") {
        s/<br>/ | /g;
      }
      print $_;

    }
  }  
   
  close (FILE);
  if ($config{'menu'} eq "side") {
    print "</td></tr></table>";
    print "</td><td valign=top>";
  }
  else {
    print "</td></tr><tr><td><img src=\"$config{'images'}/hr.gif\">";
    print "</td></tr></table>";
    #print "<table border=0 width=100%><td valign=top>";
  }

}


###########################################################################
# table_header . Prints the table header bar that most pages use.
# Usage: &table_header("Header 1");
###########################################################################
sub table_header {

  my $header = shift;
  print qq~
  <table border=0 width=100% bgcolor=$config{'tableheader'}>
    <tr><td class="header">
    <b>
    $header
    </td></tr>
  </table>
  <p>
  ~;

}

 
###########################################################################
# send_footer - Prints out the page footer
###########################################################################
sub send_footer{

  if ($config{'menu'} eq "side") { 
    print "</td></tr></table>";
  }
  print qq~
  <center>
  <br>
  <a href="$config{'homeurl'}">$config{'urltitle'}</a>
  &nbsp;&nbsp;&nbsp;
  &nbsp;&nbsp;&nbsp;
  &nbsp;&nbsp;&nbsp;
  <a href="mailto:$config{'emailaddy'}">$config{'emailtitle'}</a>
  <br><br>
  <a href="http://www.wwwthreads.org"><font size=-1>Forums powered by WWWThreads v$VERSION</font></a>
  </center>        
  ~;

  open(FILE,"$config{'includes'}/footer.include") or &cgi_error("Unable to open footer file: $config{'includes'}/footer.include. Reason: $!");
  while (<FILE>){

    print;

  } 

  close(FILE);
  print "</BODY></HTML>";

  w3t::db_disconnect;

}

###########################################################################
# not_right- User did something wrong.
###########################################################################
sub not_right{

  my $error = shift;
  &send_header ("There was a problem!");
  &table_header ("We encountered a problem!");
  print qq~
    The server encountered an error and cannot complete your request.  The error reported was:
    </p><p>
    <i>
    $error
    </i>
    </p><p>
    This error could be something as simple as your browser not being set to accept cookies.  If you feel you this error is a problem with the server then please contact <a href="mailto:$config{'emailaddy'}">$config{'emailaddy'}</a> and let us know what exactly went wrong so it can be fixed as soon as possible.  Otherwise, please use your back button to return to the previous page and try again.
    </p>
  ~;
  &send_footer;   # Send a footer back to the browser
  w3t::exit();
}

###########################################################################
# cgi_error - System did something wrong
###########################################################################
sub cgi_error {

  my $error = shift;
  print "Content-type: text/html\n\n";
  print "System Error: $error";
  #w3t::exit();
  die;
}


###########################################################################
# start_page - Logs the user on.
###########################################################################
sub start_page {
  my ($Username, $Password, $Bypass, $info) = @_;
  my $sth = '';
  my $query = '';
  my $rows = '';


# Connect to db and get total posts and last log on.
  my $username_q = $dbh->quote($Username);
  $query = qq~
    SELECT Totalposts, Laston, Password, Sort, Display, View, PostsPer, EReplies, TextCols, TextRows
    FROM   Users
    WHERE  Username = $username_q
  ~;
  $sth = $dbh->prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth->execute()               or die "Query execute error: $DBI::errstr. Query: $query";

# Make sure logs on ok..    
  if ($sth->rows < 1) {
    $sth->finish;
    w3t::not_right ("We couldn't find that Username in our database");
  }

  my ($posts, $laston, $pass, $sort, $display, $view, $postsper, $ereplies, $textcols, $textrows) = $sth->fetchrow_array;

  if ( ($Password ne $pass) && (crypt($Password,$pass) ne $pass) ) {

    w3t::send_header("Incorrect Password for that Username","<META HTTP-EQUIV=\"Refresh\" CONTENT=\"5;url=$config{'cgiurl'}/logout.pl\">");
    w3t::table_header("Incorrect password for that Username.");
    print "You will be returned to the login screen shortly.";
    w3t::send_footer();
    w3t::exit();
  }


# Set a cookie.    
  w3t::set_cookie('Username',"$Username",1,'Password',"$pass",1);

# Update last log on time.
  my $date = &get_date;
  $query = qq~
			    UPDATE Users 
			    SET    Laston = $date 
			    WHERE  Username = $username_q
  			~;
  $dbh ->do($query) or die "Query execute error: $DBI::errstr. Query: $query";

# Now give them their page.
  w3t::send_header  ("Welcome, $Username","bypass",$Username,$pass);
  w3t::table_header ("Welcome, $Username");

  my $lastlogon = &convert_time($laston);

  print qq~
    <a href="$config{'cgiurl'}/editprofile.pl"><img src="$config{'images'}/editprofile.gif" alt="Edit Profile" border=0></a>
  ~;
  if ($config{'private'}) {
    print qq~
    <a href="$config{'cgiurl'}/sendpriv.pl"><img src="$config{'images'}/sendprivate.gif" alt="Send private message" border=0></a>
    ~;
  }
  print qq~
    <a href="$config{'cgiurl'}/logout.pl"><img src="$config{'images'}/logout.gif" alt="Logout" border=0></a> 
    <p>
    Your last logon was at: <i>$lastlogon</i><br>
    Your total posts: <i>$posts</i><br><br>
    </p>
  ~;

  print "<p>$info</p>";

# Get any private messages.
  $query = qq~
    SELECT Status, Subject, Sender, Sent, Number
    FROM   Messages
    WHERE  Username = $username_q
    ORDER BY Sent DESC
  ~;
  $sth = $dbh->prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth->execute() or die "Query execute error: $DBI::errstr. Query: $query";

  $rows = $sth->rows;
  if ($rows < 1) {
    print "<p>You have no private messages.";
  }
  else {
    print qq~
      <p>You have the following private messages.
      <table border=0 width=100%>
      <tr><td bgcolor=$config{'tddark'} width=5%>&nbsp;</td>\n<td bgcolor=$config{'tddark'} width=45%>Subject</td>\n<td bgcolor=$config{'tddark'} width=20%>From</td>\n<td bgcolor=$config{'tddark'} width=25%>
Received</td>\n<td bgcolor=$config{'tddark'} width=5%>Delete</td></tr>
      <form method=post action = \"$config{'cgiurl'}/delete.pl\">
      <input type=hidden name=total value = $rows>
    ~;
    my ($read, $subject, $sender, $when, $number, $i);
      $i = 0;
    while (($read, $subject, $sender, $when, $number) = $sth->fetchrow_array) { 
      my $senton = &convert_time($when);
      if ($read eq 'N') {
        print "<tr><td bgcolor=$config{'tdlight'}>N</td>\n";
      }
      elsif ($read eq 'R') {
        print "<tr><td bgcolor=$config{'tdlight'}>R</td>\n";
      }
      else {
        print "<tr><td bgcolor=$config{'tdlight'}>&nbsp;</td>\n";
      }

      my $encsender = w3t::url_encode($sender);
      print qq~
      <td bgcolor=$config{'tdlight'}><a href="$config{'cgiurl'}/viewmessage.pl?message=$number">$subject</a></td>
      <td bgcolor=$config{'tdlight'}><a href="$config{'cgiurl'}/showprofile.pl?User=$encsender&what=login">$sender</a></td>
      <td bgcolor=$config{'tdlight'}>$senton</td>
      <td bgcolor=$config{'tdlight'}><input type=checkbox name=$i value=$number></td></tr>
      ~;
      $i++;
    }
    print "</table>";
    print "<input type = submit name = option value = \"Delete checked\">\n";
    print "</form>\n"; 
  }
  $sth->finish;
  &send_footer;

}


1;
