#!/usr/bin/perl
###########################################################################
#    WWWThreads  - Interactive Discussion Forum
#    Rick Baker scream@screamingweb.com 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
###########################################################################
  use w3tvars qw(%config);
  use w3t qw($dbh);

  use DBI;
  use strict;

  w3t::db_connect;
  my $statement = '';
  my $sth       = '';
  my $rc        = '';

##################### The Board Table ##############################

print "Creating the tables...\n";

# Create the Board Table
  $statement = qq!
CREATE TABLE Boards(
	Title Text,
	Description TEXT,
	Keyword CHAR(100) NOT NULL,
	Moderator CHAR(30),
	Total INT(9) UNSIGNED,
	Last INT(11),
	HTML CHAR(3) NOT NULL DEFAULT "On",
	Markup CHAR(3) NOT NULL DEFAULT "On",
	Anonymous CHAR(3) NOT NULL DEFAULT "YES", 
	Locked CHAR(8) NOT NULL, 
	Number INT(9) UNSIGNED DEFAULT '0' NOT NULL AUTO_INCREMENT,
        Created INT(11),
        Expire INT(4),
        Security INT(4) DEFAULT '50',
        Approve CHAR(3) NOT NULL DEFAULT "Off",
        UNIQUE indx1 (Keyword),
        INDEX indx2 (Number)
)!;

  $dbh -> do ($statement) or die "Can't execute query. Reason: $DBI::errstr";
  print "Board table created...\n";

############################## The Category Table ###################
  $statement = qq!
    CREATE TABLE Category(
      Title CHAR(120) NOT NULL DEFAULT "General Discussion",
      Number INT(9) UNSIGNED DEFAULT '0' NOT NULL,
      Keyword CHAR(100),
      Sorter INT(9) UNSIGNED DEFAULT '0',
      INDEX indx1 (Title),
      INDEX indx2 (Number)
    )
  !;
  $dbh -> do ($statement) or die "Can't execute query. Reason: $DBI::errstr.";
  print "Category table created...\n";

  $statement = qq!
    INSERT INTO Category VALUES ('General Discussion',1,'NULL',1)
  !;
  $dbh -> do ($statement) or die "Can't execute query. Reason: $DBI::errstr.";


############################## The User Table ########################

# Create the User Table
  $statement = qq!
CREATE TABLE Users(
	Username CHAR(30) NOT NULL,
	Password CHAR(20) NOT NULL,
	Email CHAR(50),
	Fakeemail CHAR(50),
        Name CHAR(100),
	FakeName CHAR(100),
	Totalposts INT(9) UNSIGNED,
	Laston INT(11) UNSIGNED,
	Signature TEXT,
	Homepage CHAR(150),
	Occupation CHAR(150),
	Hobbies CHAR(200),
	Location CHAR(200),
	Bio TEXT,
	Status CHAR(15) NOT NULL Default 'User',
        Sort INT(4),
        Display CHAR(10) NOT NULL DEFAULT "threaded",
        View CHAR(10) NOT NULL DEFAULT "collapsed",
        PostsPer INT(11),
        Number INT(9) UNSIGNED NOT NULL AUTO_INCREMENT,
        EReplies CHAR(3) NOT NULL DEFAULT "Off",
        Notify CHAR(3) NOT NULL DEFAULT "Off",
        TextCols CHAR(3),
        TextRows CHAR(3),
        Security INT(4) NOT NULL DEFAULT '50',
        FontSize CHAR(4) DEFAULT "0",
        FontFace CHAR(30) DEFAULT "",
        Extra1 CHAR(200),
        Extra2 CHAR(200),
        Extra3 CHAR(200),
        Extra4 CHAR(200),
        Extra5 CHAR(200),
        Post_Format CHAR(5) NOT NULL DEFAULT "$config{'post_format'}",
        Registered INT(12),
        Preview CHAR(5) DEFAULT "$config{'Preview'}",
        INDEX indx1 (Username, Password),
        INDEX indx2 (Status),
	INDEX indx3 (Number),
        INDEX indx4 (Security)
)!;

  $dbh -> do ($statement) or die "Can't execute query. Reason: $DBI::errstr";
  print "Users table created...\n";

######################## Private Message Table ##########################

# Create the Private Messages Table
  $statement = qq!
CREATE TABLE Messages(
	Username CHAR(30) NOT NULL,
	Status CHAR(1) NOT NULL DEFAULT 'N',
	Subject CHAR(60),
	Sender CHAR(30),
	Message TEXT,
	Sent INT(11) UNSIGNED,
	Number INT(9) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
        INDEX indx1 (Username)
)!;

  $dbh -> do ($statement) or die "Can't execute query. Reason: $DBI::errstr";
  print "Messages table created...\n";


############################ The Banned Table ###########################

  $statement = qq!
CREATE TABLE Banned(
	Username CHAR(60) NOT NULL,
	Hostname CHAR(60) NOT NULL,
	Reason TEXT,
	INDEX indx1 (Username),
        INDEX indx2 (Hostname)
)!;

  $dbh -> do ($statement) or die "Can't execute query. Reason: $DBI::errstr";
  print "Banned table created...\n";

################################### The Subscribe Table ##################
  $statement = qq!
CREATE TABLE Subscribe(
	Username CHAR(60) NOT NULL,
	Board CHAR(100) NOT NULL,
        Last INT(9) UNSIGNED,
 	INDEX indx1 (Board, Username)
)!;

  $dbh -> do ($statement) or die "Can't execute query. Reason: $DBI::errstr";
  print "Subscribe table created...\n";

# Disconnect
  w3t::db_disconnect();

print qq~

All tables created.

Don't forget to send an email to scream\@wwwthreads.org with the URL of your discussion forum, so you can be added to WWWThreads 'In Use' page.  This will give other potential Admins a chance to see the script in action.  

It will really help WWWThreads to gain in popularity if you leave the link back to wwwthreads.org intact.  If you don't feel like having it at the bottom of the forums, please link to it somewhere.

Also, if you create any custom buttons or add-ons for the program and feel like making them available to the public, please send them to me and I will put them on the miscellaneous page.

Thanks,
Scream (Rick Baker scream\@wwwthreads.org)

~;

